import { Client } from '@elastic/elasticsearch';
import { Provide, Config, Init, Inject } from '@midwayjs/core';
import { Context } from '@midwayjs/koa';
import { floor, get, head } from 'lodash';

@Provide()
export class ContactSearchService {
  @Config('elasticsearch')
  esConfig: object;
  @Inject()
  ctx: Context;
  /**es连接实例 */
  esClient: Client;
  @Init()
  async initClient() {
    const esClient = new Client(this.esConfig);
    this.esClient = esClient;
  }

  async newSearch(keyword: string, config: object, form: string, id: number) {
    const index = get(config, 'index');
    // const fields = get(config, 'fields', []);
    const user_id = get(this.ctx, 'user.id');
    const queries = [];
    // match_phrase
    queries.push({
      bool: {
        should: [{ match: { name: keyword } }],
        must: [{ term: { status: '1' } }, { term: { user: user_id } }],
        minimum_should_match: 1,
      },
    });
    // 找到这条数据
    const sourceResult = await this.esClient.search({
      index: form,
      query: { bool: { must: { match: { id } } } },
    });
    const sourceData = this.dealResponses(sourceResult);
    const industry = get(head(get(sourceData, 'data')), 'industry');
    const field = get(head(get(sourceData, 'data')), 'field');
    // if (field && field !== '') queries.push({ match: { field } });
    // if (industry && industry !== '') queries.push({ term: { industry } });
    if (field && field !== '')
      queries.push({
        bool: {
          should: [{ match: { field } }],
          must: [{ term: { status: '1' } }, { term: { user: user_id } }],
          minimum_should_match: 1,
        },
      });
    if (industry && industry !== '')
      queries.push({
        bool: {
          should: [{ term: { industry } }],
          must: [{ term: { status: '1' } }, { term: { user: user_id } }],
          minimum_should_match: 1,
        },
      });
    // 因为新加条件了:status和user,所以关系系数增加了一倍
    const result = await this.esClient.search({
      index,
      query: {
        dis_max: {
          queries,
          tie_breaker: 0.6,
          boost: 0.7,
        },
      },
      explain: true,
    });
    const returnData = this.dealResponses(result);
    const list = get(returnData, 'data', []); //最符合正向匹配的前10个数据
    // 查找,是否有推荐度为4或4以上的数据.
    const data = head(list);
    if (!data) return false;
    else {
      // 先看下正向匹配(由 浏览数据的 标题 作为关键词, 到用户审核过的数据中进行查询)
      const recommend = get(data, '_recommend', 0);
      const firRecommend = recommend >= 3;
      // 如果大于等于3,则返回成功
      if (firRecommend) return true;
    }
    return false;
  }

  /**
   * 联系申请相似度查询
   * 正反两次验证:
   *  1.正向验证: 由 查看的数据 点击 联系预约 提供的关键词 去指定索引查询并查看相似度,(取10条,但是只用第一条数据做判断,因为第一条数据是相似度最高的)
   *    再查看是否有相似度 大于等于4 的数据,如果有.则允许该联系申请
   *  2.如果正向查询的相似度 不大于等于4,则反过来用10条数据 针对 当前数据进行查询计算相似度
   * @param keyword 关键词
   * @param config 查询配置
   * @param form 联系申请是由哪里的数据发生的,数据来源:source
   * @param id  来源数据id:source_id
   * @returns
   */
  async search(keyword: string, config: object, form: string, id: number) {
    const index = get(config, 'index');
    const fields = get(config, 'fields', []);
    const user_id = get(this.ctx, 'user.id');
    // 添加条件: 当前用户, status:'1'(已审核)
    const searchObject = {
      index,
      query: {
        bool: {
          must: [
            {
              multi_match: {
                query: keyword,
                fields,
              },
            },
            {
              match_phrase: { status: '1' },
            },
            {
              match_phrase: { user: user_id },
            },
          ],
        },
      },
      from: 0,
      size: 10,
    };
    const result = await this.esClient.search(searchObject);
    const returnData = this.dealResponses(result);
    const list = get(returnData, 'data', []); //最符合正向匹配的前10个数据
    // 查找,是否有推荐度为4或4以上的数据.
    const data = head(list);
    if (!data) return false;
    else {
      // 先看下正向匹配(由 浏览数据的 标题 作为关键词, 到用户审核过的数据中进行查询)
      const recommend = get(data, '_recommend', 0);
      const firRecommend = recommend >= 4;
      // 如果大于等于4,则返回成功
      if (firRecommend) return true;
      let secRecommend = false;
      // 如果不大于等于4,再依次使用查出来的结果,反向查询一下相似度
      for (const i of list) {
        const sourceKeyword = get(i, 'name');
        const upDownSearchObject = {
          index: form,
          query: {
            bool: {
              must: [
                {
                  multi_match: {
                    query: sourceKeyword,
                    fields,
                  },
                },
                {
                  match_phrase: { status: '1' },
                },
                {
                  match_phrase: { id },
                },
              ],
            },
          },
          from: 0,
          size: 1,
        };
        const fromResult = await this.esClient.search(upDownSearchObject);
        const fromResponse = this.dealResponses(fromResult);
        const fromList = get(fromResponse, 'data', []);
        const fromData = head(fromList);
        if (!fromData) continue;
        const fromRecommend = get(fromData, '_recommend', 0);
        if (fromRecommend >= 4) {
          secRecommend = true;
          break;
        }
      }
      return secRecommend;
    }
  }

  dealResponses(result) {
    const total = get(result, 'hits.total.value', 0);
    const hits = get(result, 'hits.hits', []);
    const list = [];
    for (const ol of hits) {
      const _score = get(ol, '_score', 0);
      const data = get(ol, '_source', {});
      let recommend = 0;
      /**推荐星数计算:超过5,就都是5颗星.未超过5的都向下取整 */
      if (_score >= 5) recommend = 5;
      else recommend = floor(_score);
      data._recommend = recommend;
      list.push(data);
    }
    return { total, data: list };
  }
}
